
#ifdef __MINGW32__
#define WINVER 0x0500
#endif

#include <windows.h>

#include "dt_mdep.h"
#include "dl_defs.h"

/*** This uses the DC interface which was introduced in Win2K.
     MSVC6.0 complains that the WINVER is still beta, so it's
     skipped for MSVC. ***/

#define DC_BRUSH 18
#define DC_PEN   19

extern __declspec(dllimport) COLORREF __stdcall
   SetDCPenColor(HDC, COLORREF);
extern __declspec(dllimport) COLORREF __stdcall
   SetDCBrushColor(HDC, COLORREF); 


#ifdef __cplusplus
extern "C" {
#endif

/*** Internal prototypes ***/

DL_POINT factor11(DL_RECT *fworld, RECT *ftheWindow);

#define XPIX(r) ((int)(0.5 + theWindow.left + fact.x * ((r) - world.xmin)))
#define YPIX(r) ((int)(0.5 + theWindow.top  + fact.y * (world.ymax - (r))))

static DL_POINT fact;
static HDC hDC;
static RECT theWindow;
static DL_RECT world;

void DX_API_PUBLIC dl_my_init(HDC hdc, RECT rect, int fillbg, int bgcolor,
                              int fgcolor)
{
  theWindow = rect;
  hDC = hdc;

  /*** If desired, fill the background first. ***/

  if (fillbg)
    {
      SelectObject(hDC, GetStockObject(DC_BRUSH));
      SetDCBrushColor(hDC, bgcolor);
      FillRect(hDC, &rect, GetStockObject(DC_BRUSH));
    }

  SelectObject(hDC, GetStockObject(DC_PEN));
  /*** Set the default pen color to opposite of the background ***/
  SetDCPenColor(hDC, fgcolor);

  return;
}

void DX_API_PUBLIC dl_drawbox(DL_RECT *r)
{ 
  RECT rect;
  SetRect(&rect, XPIX(r->xmin), YPIX(r->ymax), XPIX(r->xmax), YPIX(r->ymin));
  FrameRect(hDC, &rect, GetStockObject(DC_PEN));
  return;
}

void DX_API_PUBLIC dl_drawoval(DL_RECT *r)
{
  Ellipse(hDC, XPIX(r->xmin), YPIX(r->ymax), XPIX(r->xmax), YPIX(r->ymin));
  return;
}

void DX_API_PUBLIC dl_drawto(DL_POINT *p)
{ 
  LineTo(hDC, XPIX(p->x), YPIX(p->y));
  return;
}

void DX_API_PUBLIC dl_moveto(DL_POINT *p)
{
  MoveToEx(hDC, XPIX(p->x), YPIX(p->y), NULL);
  return;
}

float DX_API_PUBLIC dl_paspect(void)
{ return 1.0; }

DL_RECT DX_API_PUBLIC dl_bounds(void)
{ return world; }

void DX_API_PUBLIC dl_fitbounds(DL_RECT *r)
{ 
  world.xmin = r->xmin;
  world.xmax = r->xmax;
  world.ymin = r->ymin;
  world.ymax = r->ymax;

  fact  = factor11(&world, &theWindow);
  return;
}

void DX_API_PUBLIC dl_flush(void)
{ return; }

void DX_API_PUBLIC dl_setga(int i)
{
  SetDCPenColor(hDC, i);
  return;
}

/*===========================================================================
 *  factor11() -- Obtain fworld-to-pixel scaling factors for 1:1 aspect ratio.
 */

DL_POINT factor11(DL_RECT *fworld, RECT *ftheWindow)
{
    DL_POINT fact, wcenter, wrange, prange;
    dt_Real  paspect, waspect, raspect;
   
   /*** Compute physical range and aspect ratio. ***/

    prange.x  = ftheWindow->right  - ftheWindow->left;
    prange.y  = ftheWindow->bottom - ftheWindow->top;
    paspect   = prange.y / prange.x;

   /*** Compute fworld center, range, and aspect ratio. ***/

    wcenter.x = (dt_Real)(fworld->xmax + fworld->xmin) / 2.0f;
    wcenter.y = (dt_Real)(fworld->ymax + fworld->ymin) / 2.0f;
    wrange.x  =  fworld->xmax - fworld->xmin;
    wrange.y  =  fworld->ymax - fworld->ymin;
    waspect   =  wrange.y / wrange.x;

   /*** Adjust fworld range so waspect matches paspect. ***/

    raspect = paspect / waspect;
    if (raspect < 1.0)
      wrange.x = wrange.x / raspect;
    if (raspect > 1.0)
      wrange.y = wrange.y * raspect;

   /*** Adjust fworld to match. ***/

    fworld->xmin = wcenter.x - wrange.x / 2.0f;
    fworld->xmax = wcenter.x + wrange.x / 2.0f;
    fworld->ymin = wcenter.y - wrange.y / 2.0f;
    fworld->ymax = wcenter.y + wrange.y / 2.0f;

   /*** Compute and return fworld-to-pixel scaling factors. ***/

    fact.x = ( wrange.x == 0 ? 1.0f : prange.x / wrange.x );
    fact.y = ( wrange.y == 0 ? 1.0f : prange.y / wrange.y );

    return fact;
}

#ifdef __cplusplus
}
#endif
